%% Chapter 10: Financial Time series
% Example: Using Time Series to Predict Equity Return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all
close all
%% Step 1. Load the data
load predict_ret_data.mat
%% Step 2. Create Financial Time Series objects
t0 = fints(sdates, sdata, {'Close'}, 'd', 'Inc');
d0 = fints(divdates, divdata, {'Dividends'}, 'u', 'Inc');
x0 = fints(expdates, expdata, {'Metric'}, 'w', 'Index');
%% Step 3. Create closing prices adjustment series
dadj1 = d0;
dadj1.dates = dadj1.dates-1;
dadj2 = d0;
dadj2.Dividends = 0;
dadj2 = fillts(dadj2,'linear','12/31/98');
dadj2('12/31/98') = 0;
dadj3 = [dadj1; dadj2];
dadj3 = fillts(dadj3, 'linear', t0.dates);
%% Step 4. Adjust closing prices and make them spot prices
t0.Spot = t0.Close - fts2mat(dadj3(datestr(t0.dates)));
%% Step 5. Create return series
tret = (t0.Spot - lagts(t0.Spot, 1)) ./ lagts(t0.Spot, 1);
tret = chfield(tret, 'Spot', 'Return');
%% Step 6. Regress return series against metric data
x1 = todaily(x0);
x1.Const = 1;
dcommon = intersect(tret.dates, x1.dates);
regts0 = [tret(datestr(dcommon)), x1(datestr(dcommon))];
finite_regts0 = find(all(isfinite( fts2mat(regts0)), 2));
regts1 = regts0( finite_regts0 );
DataMatrix = fts2mat(regts1);
XCoeff = DataMatrix(:, 2:3) \ DataMatrix(:, 1);
%XCoeff=regress(DataMatrix(:, 1),DataMatrix(:, 2:3));
RetPred = DataMatrix(:,2:3) * XCoeff;
tret.PredReturn(datestr(regts1.dates)) = RetPred;
%% Step 7. Plot the results
subplot(2, 1, 1);
plot(t0);
title('Spot and Closing Prices of Stock');
subplot(2, 1, 2);
plot(tret);
title('Actual and Predicted Return of Stock');
%% Step 8. Calculate the dividend rate
 t1 = fillts(t0,'nearest',d0.dates);
 t1.freq = 'd'; 
 tdr = d0./fts2mat(t1.Close(datestr(d0.dates)))
 
