%% Chapter 6: Fixed Income
% Example1: Sensitivity of Bond Prices to Interest Rates
%%% This example demonstrates a way to analyze the relative importance of
%%% duration and convexity for a bond portfolio
%% Step 1
Settle= '19-Aug-2009';
Maturity= ['17-Jun-2020'; '09-Jun-2025'; '14-May-2035'];
Face= [100; 100; 1000];
CouponRate= [0.07; 0.06; 0.045];
Yields = [0.05; 0.06; 0.065];
%% Step2
[CleanPrice, AccruedInterest] = bndprice(Yields, CouponRate, Settle, Maturity, 2, 0, [], [], [], [], [], Face);
Durations = bnddury(Yields, CouponRate, Settle, Maturity, 2, 0, [], [], [], [], [], Face);
Convexities = bndconvy(Yields, CouponRate, Settle, Maturity, 2, 0,[], [], [], [], [], Face);
Prices = CleanPrice + AccruedInterest;
%% Step3
dY = 0.002;
PortfolioPrice = 100000;
PortfolioWeights = ones (3,1)/3;
PortfolioAmounts = PortfolioPrice * PortfolioWeights./ Prices;
%% Step4
PortfolioDuration  = PortfolioWeights' * Durations;
PortfolioConvexity = PortfolioWeights' * Convexities;
PercentApprox1 = -PortfolioDuration * dY * 100;
PercentApprox2 =  PercentApprox1 + PortfolioConvexity*dY^2*100/2.0;
%% Step5
PriceApprox1=PortfolioPrice + PercentApprox1 * PortfolioPrice/100; 
PriceApprox2=PortfolioPrice + PercentApprox2 * PortfolioPrice/100;
%% Step 6
[CleanPrice, AccruedInterest] = bndprice(Yields + dY, CouponRate, Settle, Maturity, 2, 0, [], [], [], [], [], Face);
NewPrice = PortfolioAmounts' * (CleanPrice + AccruedInterest);
format shortg
PriceApprox1
PriceApprox2
NewPrice

