%% Chapter 6: Fixed Income
% Example 2: Bond Portfolio for Hedging Duration and Convexity
%%% This example constructs a bond portfolio to hedge the portfolio of
%%% example 1 (Sensitivity of Bond Prices to Interest Rates)

%% Step1
Settle= '19-Aug-2009';
Maturity= ['15-Jun-2015'; '02-Oct-2020'; '01-Mar-2035'];
Face= [500; 1000; 250];
CouponRate=[0.07; 0.066; 0.08];
Yields = [0.06; 0.07; 0.075];
%% Step 2
[CleanPrice, AccruedInterest] = bndprice(Yields, CouponRate, Settle, Maturity, 2, 0, [], [], [], [], [], Face);
Durations = bnddury(Yields, CouponRate, Settle, Maturity, 2, 0, [], [], [], [], [], Face);
Convexities = bndconvy(Yields, CouponRate, Settle, Maturity, 2, 0,[], [], [], [], [], Face);
Prices  =  CleanPrice + AccruedInterest;
%% Step 3
A = [Durations'
     Convexities'
     1 1 1];
b = [ 10.3181
     157.6346
       1];
Weights = A\b;
%% Step 4
PortfolioDuration  = Weights' * Durations;
PortfolioConvexity = Weights' * Convexities;
%% Step 5
PortfolioValue = 100000;
HedgeAmounts=Weights ./ Prices * PortfolioValue;
format shortg
PortfolioDuration
PortfolioConvexity
HedgeAmounts