%% Chapter 6: Fixed Income
% Example 3: Bond Prices and Yield Curve Parallel Shifts
%%% This example uses Financial Toolbox bond pricing functions to evaluate the impact of time-to-maturity and yield variation on the price of a bond portfolio
%% Step 1
Settle='15-Jan-2015';
Maturity= datenum(['03-Apr-2040'; '14-May-2045'; '09-Jun-2039'; '25-Feb-2039']);
Face = [1000; 1000; 1000; 1000];
CouponRate= [0; 0.05; 0; 0.055];
Periods= [0; 2; 0; 2];
Yields = [0.078; 0.09; 0.075; 0.085];
%% Step 2
[CleanPrice, AccruedInterest] = bndprice(Yields,... 
CouponRate,Settle, Maturity, Periods,...
[], [], [], [], [], [], Face);
Prices  =  CleanPrice + AccruedInterest;
%% Step 3
BondAmounts = 25000 ./ Prices;
%% Step 4
dy = -0.05:0.005:0.05;               % Yield changes
D  = datevec(Settle);                % Get date components
dt = datenum(D(1):2038, D(2), D(3)); % Get evaluation dates
[dT, dY]  =  meshgrid(dt, dy); % Create grid
NumTimes  =  length(dt);       % Number of time steps
NumYields =  length(dy);       % Number of yield changes
NumBonds  =  length(Maturity); % Number of bonds
% Preallocate vector
Prices = zeros(NumTimes*NumYields, NumBonds);
for i = 1:NumBonds
   [CleanPrice, AccruedInterest] = bndprice(Yields(i)+... 
   dY(:), CouponRate(i), dT(:), Maturity(i), Periods(i),...
   [], [], [], [], [], [], Face(i));
   Prices(:,i) = CleanPrice + AccruedInterest;
end
Prices = Prices * BondAmounts;
Prices = reshape(Prices, NumYields, NumTimes);
%% Step 5
figure                   % Open a new figure window
surf(dt, dy, Prices)     % Draw the surface
hold on                  % Add the current value for reference
basemesh = mesh(dt, dy, 100000*ones(NumYields, NumTimes));
set(basemesh, 'facecolor', 'none');
set(basemesh, 'edgecolor', 'm');
set(gca, 'box', 'on');
dateaxis('x', 11);
xlabel('Evaluation Date (YY Format)');
ylabel('Change in Yield');
zlabel('Portfolio Price');
hold off
view(-25,25);