%% Chapter 6: Fixed Income
% Example 4: Bond Prices and Yield Curve Nonparallel Shifts
%%% This example uses bndkrdur to construct a portfolio to hedge the interest rate risk of a U.S. Treasury bond maturing in 20 years.

%% characteristics of bond
%U.S. Treasury bond
Settle = datenum('2-Dec-2008');
CouponRate = 5.500/100;
Maturity = datenum('15-Aug-2028');
Price = 128.68;
%The 30-year bond
Maturity_30 = datenum('15-May-2038');
Coupon_30 = .045;
Price_30 = 124.69;
%The 10-year note
Maturity_10 = datenum('15-Nov-2018');
Coupon_10 = .0375;
Price_10 = 109.35;
%The five-year note
Maturity_05 = datenum('30-Nov-2013');
Coupon_05 = .02;
Price_05 = 101.67;
%The two-year note
Maturity_02 = datenum('30-Nov-2010');
Coupon_02 = .01250;
Price_02 = 100.72;
%% Step1
ZeroDates = daysadd(Settle,[30 90 180 360 360*2 360*3 360*5 360*7 360*10 360*20 360*30]);
ZeroRates = ([0.09 0.07 0.44 0.81 0.90 1.16 1.71 2.13 2.72 3.51 3.22]/100)';
BondKRD = bndkrdur ([ZeroDates ZeroRates], CouponRate, Settle, Maturity,'keyrates',[2 5 10 20]);
HedgeMaturity = [Maturity_02; Maturity_05; Maturity_10; Maturity_30];
HedgeCoupon = [Coupon_02; Coupon_05; Coupon_10; Coupon_30];
HedgeKRD = bndkrdur ([ZeroDates ZeroRates], HedgeCoupon, Settle, HedgeMaturity, 'keyrates', [2 5 10 20]);
%% Step 2
PortfolioDD = 100*Price* BondKRD;
HedgeDD = bsxfun (@times, HedgeKRD, [Price_02; Price_05; Price_10; Price_30]);
%% Step3
NumBonds = PortfolioDD/HedgeDD


