%% Chapter 7: Options
%Example 1: Greek-Neutral Portfolios of European Stock Options

%% Step 1
DataMatrix = [100.000 100 0.2 0.3 0 % Call
                119.100 125 0.2 0.2 0.025 % Put
                87.200 85 0.1 0.23 0 % Call
                301.125 315 0.5 0.25 0.0333]; % Put
RiskFreeRate = 0.10;
StockPrice = DataMatrix(:,1);
StrikePrice = DataMatrix(:,2);
ExpiryTime = DataMatrix(:,3);
Volatility = DataMatrix(:,4);
DividendRate = DataMatrix(:,5);
%% Step 2
[CallPrices, PutPrices] = blsprice(StockPrice, StrikePrice, RiskFreeRate, ExpiryTime, Volatility, DividendRate);
[CallDeltas, PutDeltas] = blsdelta(StockPrice, StrikePrice, RiskFreeRate, ExpiryTime, Volatility, DividendRate);
Gammas = blsgamma(StockPrice, StrikePrice, RiskFreeRate, ExpiryTime, Volatility , DividendRate)';
Vegas = blsvega(StockPrice, StrikePrice, RiskFreeRate, ExpiryTime, Volatility , DividendRate)';
Prices = [CallPrices(1) PutPrices(2) CallPrices(3) PutPrices(4)];
Deltas = [CallDeltas(1) PutDeltas(2) CallDeltas(3) PutDeltas(4)];
%% Step 3
A = [Deltas; Gammas; Vegas; Prices];
b = [0; 0; 0; 17000]; 
OptionQuantities = A\b; 
%% Step 4
PortfolioValue = Prices * OptionQuantities;
PortfolioDelta = Deltas * OptionQuantities;
PortfolioGamma = Gammas * OptionQuantities;
PortfolioVega = Vegas * OptionQuantities;
