%% Chapter 9: Credit Risk
% Example 1: Forecasting Corporate Default Rates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Part I: Working with Credit Migrations Data
clc
clear
close all
Example_LoadData
% Getting Default Rates for Different Ratings Categories
totalsByRtg(nYears,1) = struct('totalsVec',[],'totalsMat',[],...
'algorithm','cohort');
for t = 1:nYears
    totalsByRtg(t).totalsVec = nIssuers(t,:);
    totalsByRtg(t).totalsMat = round(diag(nIssuers(t,:))*...
    (0.01*TransMat(:,:,t)));
    totalsByRtg(t).algorithm = 'cohort';
end
fprintf('\nTransition matrix for 2005:\n\n')
Example_DisplayTransitions(TransMat(:,:,end),nIssuers(end,:),...
{'AAA','AA','A','BBB','BB','B','CCC'},...
{'AAA','AA','A','BBB','BB','B','CCC','D','NR'})

fprintf('\nTransition counts (totals struct) for 2005:\n\n')
Example_DisplayTransitions(totalsByRtg(end).totalsMat,...
totalsByRtg(end).totalsVec,...
{'AAA','AA','A','BBB','BB','B','CCC'},...
{'AAA','AA','A','BBB','BB','B','CCC','D','NR'})

edgesIGSG = [4 7];
totalsIGSG = transprobgrouptotals(totalsByRtg,edgesIGSG);
edgesAll = 7; 
totalsAll = transprobgrouptotals(totalsByRtg,edgesAll);

fprintf('\nTransition counts for 2005 at IG/SG level:\n\n')
Example_DisplayTransitions(totalsIGSG(end).totalsMat,...
totalsIGSG(end).totalsVec,{'IG','SG'},{'IG','SG','D','NR'})

fprintf('\nTransition matrix for 2005 at IG/SG level:\n\n')
Example_DisplayTransitions(transprobbytotals(totalsIGSG(end)),[],...
{'IG','SG'},{'IG','SG','D','NR'})

DefRateIG = zeros(nYears,1);
DefRateSG = zeros(nYears,1);
DefRate = zeros(nYears,1);
for t=1:nYears
    tmIGSG = transprobbytotals(totalsIGSG(t));
    DefRateIG(t) = tmIGSG(1,3);
    DefRateSG(t) = tmIGSG(2,3);
    tmAll = transprobbytotals(totalsAll(t));
    DefRate(t) = tmAll(1,2);
end

minIG = min(DefRateIG(DefRateIG~=0));
figure
plot(Years,log(DefRateSG),'m-*')
hold on
plot(Years,log(DefRate),'b-o')
plot(Years,log(max(DefRateIG,minIG-0.001)),'r-+')
Example_RecessionBands
hold off
grid on
title('{Default Rates (log scale)}')
ylabel('log %')
legend({'SG','Overall','IG'},'location','NW')

% Getting Default Rates for Different Time Periods
tmAllRec = transprobbytotals(totalsAll(Recession));
DefRateRec = tmAllRec(1,2);
tmAllExp = transprobbytotals(totalsAll(~Recession));
DefRateExp = tmAllExp(1,2);
tmAllTTC = transprobbytotals(totalsAll);
DefRateTTC = tmAllTTC(1,2);
DefRateTwoValues = DefRateExp*ones(nYears,1);
DefRateTwoValues(Recession) = DefRateRec;
figure
plot(Years,DefRate,'bo:','LineWidth',1.2)
hold on
stairs(Years-0.5,DefRateTwoValues,'m-','LineWidth',1.5)
plot(Years,DefRateTTC*ones(nYears,1),'r-.','LineWidth',1.5)
Example_RecessionBands
hold off
grid on
title('{Default Rate}')
ylabel('%')
legend({'Point-in-time (PIT)','Recession/Expansion Avg',...
'Through-the-cycle (TTC)'},'location','NW')
% Building Predictors Using Credit Ratings Data
nEOY = zeros(nYears,1);
nEOY(1:end-1) = sum(nIssuers(2:end,:),2);
nEOY(end) = totalsAll(end).totalsMat(1,1) + sum(nNewIssuers(end,:));
AGE = 100*[nan(3,1); sum(nNewIssuers(1:end-3,:),2)./nEOY(4:end)];

indSG = 5:7;
nSGEOY = zeros(nYears,1);
nSGEOY(1:end-1) = sum(nIssuers(2:end,indSG),2);
nSGEOY(end) = sum(totalsIGSG(end).totalsMat(:,2)) +...
sum(nNewIssuers(end,indSG));
SG = 100*nSGEOY./nEOY;
AGESG = 100*[nan(3,1); sum(nNewIssuers(1:end-3,indSG),2)./nEOY(4:end)];
%% Part II: A Forecasting Model for Default Rates%%%
inSample = 4:nYears-1;
T = length(inSample);
varNames = {'AGE','CPF','SPR'};
X = [AGE CPF SPR];
X = X(inSample,:);
y = DefRate(inSample+1); % DefaultRate, year t+1
stats = regstats(y,X);
fprintf('\nConst AGE CPF SPR adjR^2\n')
fprintf('%1.2f %1.2f %1.2f %1.2f %1.4f\n',...
[stats.beta;stats.adjrsquare])
yHat=stats.yhat;
bHat = stats.beta;
yHat2 = [ones(T,1),X]*bHat;
figure
plot(Years(inSample+1),DefRate(inSample+1),'ko','LineWidth',1.5,...
'MarkerSize',10,'MarkerFaceColor','g')
hold on
plot(Years(inSample+1),yHat,'b-s','LineWidth',1.2,'MarkerSize',10)
hold off
grid on
legend({'Actual','Model'},'location','NW')
title('{\bf Corporate Default Rate Models: In-Sample Fit}')
xlabel('Year')
ylabel('Percent')

% Backtesting
XBT = [AGE,CPF,SPR];
yBT = DefRate;
iYear0 = find(Years==1984);
T = find(Years==1995); 
YearsBT = 1996:2005; 
iYearsBT = find(Years==1996):find(Years==2005); 
nYearsBT = length(YearsBT); 
MethodTags = {'Model','PIT','TTC'};
nMethods = length(MethodTags);
PredDefRate = zeros(nYearsBT,nMethods);
ErrorBT = zeros(nYearsBT,nMethods);
alpha = 0.05;
PredDefLoBnd = zeros(nYearsBT,1);
PredDefUpBnd = zeros(nYearsBT,1);
for k=1:nYearsBT
    inSampleBT = iYear0:T-1;
    s = regstats(yBT(inSampleBT+1),XBT(inSampleBT,:));
    PredDefRate(k,1) = [1 XBT(T,:)]*s.beta;
    tCrit = tinv(1-alpha/2,s.tstat.dfe);
    PredStd = sqrt([1 XBT(T,:)]*s.covb*[1 XBT(T,:)]'+s.mse);
    PredDefLoBnd(k) = max(0,PredDefRate(k,1) - tCrit*PredStd);
    PredDefUpBnd(k) = PredDefRate(k,1) + tCrit*PredStd;
    PredDefRate(k,2) = DefRate(T);
    tmAll = transprobbytotals(totalsAll(iYear0:T));
    PredDefRate(k,3) = tmAll(1,2);
    ErrorBT(k,:) = PredDefRate(k,:) - DefRate(T+1);
    T = T + 1;
end
Example_BacktestPlot(YearsBT,DefRate(iYearsBT),PredDefRate,'Year','%',...
'{Default Rate Estimation Methods: Backtesting}',['Actual' MethodTags],'NW')

CumSqError = cumsum(ErrorBT.^2);
Example_BacktestPlot(YearsBT,[],CumSqError,'Year','Cum Sq Error',...
'{Cumulative Square Error in Backtesting Exercise}',MethodTags,'NW')

EAD = 100*ones(nYearsBT,1); 
LGD = 0.45*ones(nYearsBT,1);
ReservesExcessShortfall = bsxfun(@times,EAD.*LGD,ErrorBT/100);
%EL=bsxfun(@times,EAD.*LGD,PredDefRate/100);
%AL=bsxfun(@times,EAD.*LGD,DefRate(end-9:end,1)/100);
%ReservesExcessShortfall=bsxfun(@minus,EL,AL)
ReservesBalanceEOY = cumsum(ReservesExcessShortfall);
Example_BacktestPlot(YearsBT,[],ReservesBalanceEOY,'Year','Billions of Dollars',...
'{Reserves Balance (EOY): Backtesting}',MethodTags,'SW')
grid on

figure
plot(YearsBT,DefRate(iYearsBT),'ko','LineWidth',1.5,'MarkerSize',10,...
'MarkerFaceColor','g')
hold on
plot(YearsBT,PredDefRate(:,1),'b-s','LineWidth',1.2,'MarkerSize',10)
plot(YearsBT,[PredDefLoBnd PredDefUpBnd],'b:','LineWidth',1.2)
hold off
strConf = num2str((1-alpha)*100);
title(['{Backtesting Results with ' strConf '% Prediction Intervals}'])
xlabel('Year');
ylabel('%');
legend({'Actual','Predicted','Conf Bounds'},'location','NW');

% Stress Testing
tCrit = tinv(1-alpha/2,stats.tstat.dfe);
XLast = [AGE(end),CPF(end),SPR(end)];
yPred = [1 XLast]*stats.beta;
PredStd = sqrt([1 XLast]*stats.covb*[1 XLast]'+stats.mse);
yPredUB = yPred + tCrit*PredStd;
fprintf('\nPredicted default rate:\n');
fprintf('	 	Baseline: %4.2f%%\n',yPred);
fprintf(' 		%g%% Upper Bound: %4.2f%%\n',(1-alpha)*100,yPredUB);
yPredFn = @(cpf,spr) [1 AGE(end) cpf spr]*stats.beta;
PredStdFn = @(cpf,spr) sqrt([1 AGE(end) cpf spr]*stats.covb*...
[1 AGE(end) cpf spr]'+stats.mse);
yPredUBFn = @(cpf,spr) (yPredFn(cpf,spr) + tCrit*PredStdFn(cpf,spr));
yPredLBFn = @(cpf,spr) (yPredFn(cpf,spr) - tCrit*PredStdFn(cpf,spr));
fprintf('\n\n		 What-if Analysis\n');
fprintf('Scenario		 LB 	Pred 		UB\n');
cpf = CPF(end)*0.96;
spr = SPR(end);
yPredRange = [yPredLBFn(cpf,spr),yPredFn(cpf,spr),yPredUBFn(cpf,spr)];
fprintf('CPF drops 4%% 	%4.2f%% 	%4.2f%% 	%4.2f%%\n',yPredRange);
cpf = CPF(end);
spr = SPR(end)+1;
yPredRange = [yPredLBFn(cpf,spr),yPredFn(cpf,spr),yPredUBFn(cpf,spr)];
fprintf('SPR rises 1%%	%4.2f%% 	%4.2f%% 	%4.2f%%\n',yPredRange);
cpf = CPF(end);
spr = SPR(end);
yPredRange = [yPredLBFn(cpf,spr),yPredFn(cpf,spr),yPredUBFn(cpf,spr)];
fprintf(' Baseline      	%4.2f%%	%4.2f%%   %4.2f%%\n',yPredRange);
fprintf('\nCorrelation between CPF and SPR: %4.3f\n',corr(CPF,SPR));

gridCPF = 2*min(CPF):0.1:max(CPF);
gridSPR = min(SPR):0.1:2*max(SPR);
nGridCPF = length(gridCPF);
nGridSPR = length(gridSPR);
DefRateUB = zeros(nGridCPF,nGridSPR);
for i=1:nGridCPF
    for j=1:nGridSPR
        DefRateUB(i,j) = yPredUBFn(gridCPF(i),gridSPR(j));
    end
end
Example_StressTestPlot(gridCPF,gridSPR,DefRateUB,CPF,SPR,...
    'Corporate Profits Forecast (%)','Corporate Spread (%)',...
    ['{' strConf '% UB Default Rate Regions (in %)}'])


w = @(pd) (1-exp(-50*pd))/(1-exp(-50)); 
R = @(pd) (0.12*w(pd)+0.24*(1-w(pd))); 
V = @(pd) normcdf(norminv(pd)+R(pd).*norminv(0.999)./sqrt(1-R(pd)));
b = @(pd) (0.11852-0.05478*log(pd)).^2;
K = @(pd) 0.45*(V(pd)-pd).*(1./(1-1.5*b(pd)));
CapReq = 100*K(DefRateUB/100);
Example_StressTestPlot(gridCPF,gridSPR,CapReq,CPF,SPR,...
'Corporate Profits Forecast (%)','Corporate Spread (%)',...
{'{\bf Capital Requirement Regions (% of value)}';...
['{\bf using ' strConf '% UB Default Rate}']})

