%% Chapter 9: Credit Risk
% Example2: Case Study for a Credit Scorecard Analysis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all
close all
%% Step 1. Create a creditscorecard object
load CreditCardData
sc = creditscorecard(data,'IDVar','CustID')
bininfo(sc,'ResStatus')
plotbins(sc,'ResStatus')
cp = 20000:5000:60000;
sc = modifybins(sc,'CustIncome','CutPoints',cp);
bininfo(sc,'CustIncome')
plotbins(sc,'CustIncome')
%% Step 2a. Automatically bin the data.
sc = autobinning(sc);
plotbins(sc,sc.PredictorVars)
%Step 2b. Fine-tune the bins using manual binning.
[bi,cp] = bininfo(sc,'CustAge');
cp([1 5]) = []; % To merge bins 1 and 2, and bins 5 and 6
sc = modifybins(sc,'CustAge','CutPoints',cp);
plotbins(sc,'CustAge')
[bi,cp] = bininfo(sc,'CustIncome');
cp([3 4]) = [];
sc = modifybins(sc,'CustIncome','CutPoints',cp);
plotbins(sc,'CustIncome')
[bi,cp] = bininfo(sc,'TmWBank');
cp(2) = [];
sc = modifybins(sc,'TmWBank','CutPoints',cp);
plotbins(sc,'TmWBank')
[bi,cp] = bininfo(sc,'AMBalance');
cp(2) = [];
sc = modifybins(sc,'AMBalance','CutPoints',cp);
plotbins(sc,'AMBalance')
%% Step 3. Fit a logistic regression model.
sc = fitmodel(sc);
%% Step 4. Review and format scorecard points.
p1 = displaypoints(sc);
disp(p1)
sc = modifybins(sc,'CustAge','BinLabels',...
{'Up to 36' '37 to 39' '40 to 45' '46 to 57' '58 and up'});
sc = modifybins(sc,'CustIncome','BinLabels',...
{'Up to 28999' '29000 to 32999' '33000 to 41999' '42000 to 46999' '47000 and up'});
sc = modifybins(sc,'TmWBank','BinLabels',...
{'Up to 11' '12 to 44' '45 to 70' '71 and up'});
sc = modifybins(sc,'AMBalance','BinLabels',...
{'Up to 558.87' '558.88 to 1597.43' '1597.44 and up'});
p1 = displaypoints(sc);
disp(p1)
TargetPoints = 500;
TargetOdds = 2;
PDO = 50; % Points to double the odds
sc = formatpoints(sc,'PointsOddsAndPDO',[TargetPoints TargetOdds PDO],'Round','AllPoints');
p2 = displaypoints(sc);
disp(p2)
%% Step 5. Score the data.
[Scores,Points] = score(sc);
disp(Scores(1:10))
disp(Points(1:10,:))
%% Step 6. Calculate the probability of default.
pd = probdefault(sc);
ProbGood = 1-pd;
PredictedOdds = ProbGood./pd;
figure
scatter(Scores,PredictedOdds)
title('Predicted Odds vs. Score')
xlabel('Score')
ylabel('Predicted Odds')
hold on
xLimits = xlim;
yLimits = ylim;
        % Target points and odds
plot([TargetPoints TargetPoints],[yLimits(1) TargetOdds],'k:')
plot([xLimits(1) TargetPoints],[TargetOdds TargetOdds],'k:')
        % Target points plus PDO
plot([TargetPoints+PDO TargetPoints+PDO],[yLimits(1) 2*TargetOdds],'k:')
plot([xLimits(1) TargetPoints+PDO],[2*TargetOdds 2*TargetOdds],'k:')
        % Target points minus PDO
plot([TargetPoints-PDO TargetPoints-PDO],[yLimits(1) TargetOdds/2],'k:')
plot([xLimits(1) TargetPoints-PDO],[TargetOdds/2 TargetOdds/2],'k:')
hold off
%% Step 7. Validate the credit scorecard model using the CAP, ROC, and Kolmogorov-Smirnov statistic
[Stats,T] = validatemodel(sc,'Plot',{'CAP','ROC','KS'});
disp(Stats)
disp(T(1:5,:))
